"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sql_template_strings_1 = require("sql-template-strings");
const adapter_1 = require("../adapter");
const migration_1 = require("../migration");
class PostgresAdapter extends adapter_1.DbAdapter {
    /**
     * @param url The connection url
     * @param lib The pg library
     */
    constructor(url, lib) {
        super();
        this.lib = lib;
        this.client = new this.lib.Client(url);
    }
    /**
     * Connects to the database and sets up the schema if required
     */
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve, reject) => this.client.connect(err => err ? reject(err) : resolve()));
            try {
                yield this.client.query(`CREATE TYPE "merkel_migration_type" AS ENUM ('up', 'down')`);
            }
            catch (err) {
                /* istanbul ignore next */
                if (~~err.code !== 42710) {
                    throw err;
                }
                // else ignore
            }
            yield this.client.query(`
            CREATE TABLE IF NOT EXISTS "merkel_meta" (
                "id" SERIAL NOT NULL PRIMARY KEY,
                "name" TEXT NOT NULL,
                "type" merkel_migration_type,
                "commit" TEXT,
                "head" TEXT NOT NULL,
                "applied_at" TIMESTAMP WITH TIME ZONE NOT NULL
            );
        `);
        });
    }
    close() {
        return new Promise((resolve, reject) => {
            this.client.on('end', resolve);
            this.client.end();
        });
    }
    /**
     * Gets the last migration task that was executed
     */
    getLastMigrationTask() {
        return __awaiter(this, void 0, void 0, function* () {
            // find out the current database state
            const { rows } = yield this.client.query(`
            SELECT "id", "name", "applied_at", "type", "commit", "head"
            FROM "merkel_meta"
            ORDER BY "id" DESC
            LIMIT 1
        `);
            return rows.length === 0 ? null : this.rowToTask(rows[0]);
        });
    }
    /**
     * Logs an executed task to the database. Sets the task ID
     */
    logMigrationTask(task) {
        return __awaiter(this, void 0, void 0, function* () {
            const { rows } = yield this.client.query(sql_template_strings_1.SQL `
            INSERT INTO merkel_meta ("name", "type", "commit", "head", "applied_at")
            VALUES (
                ${task.migration.name},
                ${task.type},
                ${task.commit ? task.commit.sha1 : null},
                ${task.head.sha1},
                ${task.appliedAt}
            )
            RETURNING id
        `);
            task.id = rows[0]['id'];
        });
    }
    /**
     * Checks that the same task cannot be executed two times in a row and the first task cannot be
     * a down task
     */
    checkIfTaskCanExecute(task) {
        return __awaiter(this, void 0, void 0, function* () {
            const { rows } = yield this.client.query(sql_template_strings_1.SQL `
            SELECT "type"
            FROM "merkel_meta"
            WHERE "name" = ${task.migration.name}
            ORDER BY "id" DESC
            LIMIT 1
        `);
            if (task.type === 'up') {
                if (rows.length > 0 && rows[0]['type'] === 'up') {
                    throw new migration_1.MigrationRunTwiceError(task.migration, 'up');
                }
            }
            else if (task.type === 'down') {
                if (rows.length === 0) {
                    throw new migration_1.FirstDownMigrationError(task.migration);
                }
                else if (rows[0]['type'] === 'down') {
                    throw new migration_1.MigrationRunTwiceError(task.migration, 'down');
                }
            }
        });
    }
}
exports.PostgresAdapter = PostgresAdapter;
//# sourceMappingURL=postgres.js.map