"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const postgres_1 = require("../../adapters/postgres");
const migration_1 = require("../../migration");
const git_1 = require("../../git");
const pg = require("pg");
const assert = require("assert");
describe('PostgresAdapter', () => {
    let client;
    before(() => __awaiter(this, void 0, void 0, function* () {
        client = new pg.Client(process.env.MERKEL_DB);
        yield new Promise((resolve, reject) => client.connect(err => err ? reject(err) : resolve()));
    }));
    after(() => client.end());
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        yield client.query('DROP TABLE IF EXISTS "merkel_meta"');
        yield client.query('DROP TYPE IF EXISTS "merkel_migration_type"');
    }));
    describe('init', () => {
        it('should create the database schema', () => __awaiter(this, void 0, void 0, function* () {
            const adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
            yield adapter.init();
            const client = new pg.Client(process.env.MERKEL_DB);
            yield new Promise((resolve, reject) => client.connect(err => err ? reject(err) : resolve()));
            const { rows } = yield client.query(`SELECT column_name, data_type FROM INFORMATION_SCHEMA.COLUMNS WHERE table_name = 'merkel_meta' ORDER BY column_name ASC`);
            assert.deepEqual(rows, [
                { column_name: 'applied_at', data_type: 'timestamp with time zone' },
                { column_name: 'commit', data_type: 'text' },
                { column_name: 'head', data_type: 'text' },
                { column_name: 'id', data_type: 'integer' },
                { column_name: 'name', data_type: 'text' },
                { column_name: 'type', data_type: 'USER-DEFINED' }
            ]);
            client.end();
        }));
        it('should not fail initializing twice', () => __awaiter(this, void 0, void 0, function* () {
            let adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
            yield adapter.init();
            yield adapter.close();
            adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
            yield adapter.init();
        }));
    });
    describe('logMigrationTask', () => {
        it('should log migrations correctly', () => __awaiter(this, void 0, void 0, function* () {
            const adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
            yield adapter.init();
            const date = new Date(Date.now());
            const task = new migration_1.Task({
                appliedAt: date,
                type: 'up',
                migration: new migration_1.Migration({
                    name: 'testlog'
                }),
                commit: new git_1.Commit({
                    sha1: '1234'
                }),
                head: new git_1.Commit({
                    sha1: '2345'
                })
            });
            yield adapter.logMigrationTask(task);
            const test = yield adapter.getLastMigrationTask();
            assert.deepEqual(test, task);
        }));
    });
    describe('getLastMigrationTask', () => {
        it('should get the latest migration task', () => __awaiter(this, void 0, void 0, function* () {
            const adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
            yield adapter.init();
            const date = new Date(Date.now());
            adapter.logMigrationTask(new migration_1.Task({
                appliedAt: date,
                type: 'up',
                commit: new git_1.Commit({
                    sha1: '123'
                }),
                head: new git_1.Commit({
                    sha1: '234'
                }),
                migration: new migration_1.Migration({ name: 'test' })
            }));
            const task = yield adapter.getLastMigrationTask();
            assert.deepEqual(task, {
                id: 1,
                appliedAt: date,
                type: 'up',
                migration: {
                    name: 'test'
                },
                commit: {
                    sha1: '123',
                    tasks: []
                },
                head: {
                    sha1: '234',
                    tasks: []
                }
            });
        }));
    });
    describe('checkIfTaskCanExecute', () => __awaiter(this, void 0, void 0, function* () {
        it('should not run the same migration up twice', () => __awaiter(this, void 0, void 0, function* () {
            const adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
            yield adapter.init();
            const upTask = new migration_1.Task({
                appliedAt: new Date(Date.now()),
                migration: new migration_1.Migration({
                    name: 'test'
                }),
                type: 'up',
                commit: new git_1.Commit({
                    sha1: '1'
                }),
                head: new git_1.Commit({
                    sha1: '2'
                })
            });
            yield adapter.logMigrationTask(upTask);
            try {
                yield adapter.checkIfTaskCanExecute(upTask);
                throw new assert.AssertionError({
                    message: 'task could be executed twice'
                });
            }
            catch (err) {
                if (!(err instanceof migration_1.MigrationRunTwiceError)) {
                    throw err;
                }
            }
        }));
        it('should not run the same migration down twice', () => __awaiter(this, void 0, void 0, function* () {
            const adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
            yield adapter.init();
            const downTask = new migration_1.Task({
                appliedAt: new Date(Date.now()),
                migration: new migration_1.Migration({
                    name: 'test'
                }),
                type: 'down',
                commit: new git_1.Commit({
                    sha1: '1'
                }),
                head: new git_1.Commit({
                    sha1: '2'
                })
            });
            yield adapter.logMigrationTask(downTask);
            try {
                yield adapter.checkIfTaskCanExecute(downTask);
                throw new assert.AssertionError({
                    message: 'task could be executed twice'
                });
            }
            catch (err) {
                if (!(err instanceof migration_1.MigrationRunTwiceError)) {
                    throw err;
                }
            }
        }));
        it('should not run a down migration first', () => __awaiter(this, void 0, void 0, function* () {
            const adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
            yield adapter.init();
            const task = new migration_1.Task({
                appliedAt: new Date(Date.now()),
                migration: new migration_1.Migration({
                    name: 'test'
                }),
                type: 'down',
                commit: new git_1.Commit({
                    sha1: '1'
                }),
                head: new git_1.Commit({
                    sha1: '2'
                })
            });
            try {
                yield adapter.checkIfTaskCanExecute(task);
                throw new assert.AssertionError({
                    message: 'down task could be executed first'
                });
            }
            catch (err) {
                if (!(err instanceof migration_1.FirstDownMigrationError)) {
                    throw err;
                }
            }
        }));
    }));
});
//# sourceMappingURL=postgres.test.js.map