"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const git_1 = require("../git");
const index_1 = require("../index");
const child_process_1 = require("mz/child_process");
const assert = require("assert");
const fs = require("mz/fs");
const path = require("path");
const del = require("del");
const os_1 = require("os");
const repo = path.join(os_1.tmpdir(), 'merkel_test_repo');
console.log(repo);
describe('git', () => {
    beforeEach(() => __awaiter(this, void 0, void 0, function* () {
        try {
            yield fs.access(repo);
        }
        catch (err) {
            yield fs.mkdir(repo);
        }
        process.chdir(repo);
        yield del('*', { dot: true });
        yield child_process_1.execFile('git', ['init']);
    }));
    describe('hook', () => {
        it('should add githook', () => __awaiter(this, void 0, void 0, function* () {
            yield git_1.addGitHook();
            const hook = yield fs.readFile(path.join(process.cwd(), '.git/hooks/prepare-commit-msg'), 'utf8');
            assert(hook.includes('merkel prepare-commit-msg'));
        }));
        it('should not add githook twice', () => __awaiter(this, void 0, void 0, function* () {
            yield git_1.addGitHook();
            try {
                yield git_1.addGitHook();
                throw new assert.AssertionError({
                    message: 'it added the git hook twice.'
                });
            }
            catch (err) {
                if (!(err instanceof git_1.HookAlreadyFoundError)) {
                    throw err;
                }
            }
        }));
        it('should append the githook, when the prepare-commit-msg hook already exists', () => __awaiter(this, void 0, void 0, function* () {
            yield fs.writeFile(path.join(process.cwd(), '.git', 'hooks', 'prepare-commit-msg'), 'keep me\n');
            yield git_1.addGitHook();
            const hook = yield fs.readFile(path.join(process.cwd(), '.git/hooks/prepare-commit-msg'), 'utf8');
            assert(hook.includes('keep me'));
        }));
    });
    describe('getConfigurationForCommit', () => {
        it('should get the configuration for every commit', () => __awaiter(this, void 0, void 0, function* () {
            yield index_1.createConfig({
                migrationDir: 'migrations',
                migrationOutDir: 'migrations'
            });
            yield child_process_1.execFile('git', ['add', '.']);
            yield child_process_1.execFile('git', ['commit', '-m', 'initial commit']);
            yield index_1.createConfig({
                migrationDir: 'src/migrations',
                migrationOutDir: 'dist/migrations'
            });
            yield child_process_1.execFile('git', ['add', '.']);
            yield child_process_1.execFile('git', ['commit', '-m', 'fixes migrations']);
            const commits = yield git_1.getNewCommits();
            let config = yield git_1.getConfigurationForCommit(commits[0]);
            assert.equal(config.migrationDir, 'migrations');
            assert.equal(config.migrationOutDir, 'migrations');
            config = yield git_1.getConfigurationForCommit(commits[1]);
            assert.equal(config.migrationDir, 'src/migrations');
            assert.equal(config.migrationOutDir, 'dist/migrations');
        }));
        it('should return null if no merkelrc is in commit', () => __awaiter(this, void 0, void 0, function* () {
            yield fs.writeFile('test.js', 'const one = 1;');
            yield child_process_1.execFile('git', ['add', '.']);
            yield child_process_1.execFile('git', ['commit', '-m', 'initial commit']);
            const head = yield git_1.getHead();
            assert.equal(yield git_1.getConfigurationForCommit(head), null);
            yield index_1.createConfig({
                migrationDir: 'migrations',
                migrationOutDir: 'migrations'
            });
            assert.equal(yield git_1.getConfigurationForCommit(head), null);
        }));
    });
    describe('getNewCommits', () => {
        it('should return all new commits', () => __awaiter(this, void 0, void 0, function* () {
            for (let i = 0; i < 3; i++) {
                yield fs.appendFile('test.txt', `${i}\n`);
                yield child_process_1.execFile('git', ['add', '.']);
                yield child_process_1.execFile('git', ['commit', '-m', `${i}`]);
            }
            const commitSequence = yield git_1.getNewCommits();
            assert.equal(commitSequence.length, 3);
            assert.equal(commitSequence.isReversed, false);
            assert.equal(commitSequence[0].message, '0');
            assert.equal(commitSequence[1].message, '1');
            assert.equal(commitSequence[2].message, '2');
        }));
        it('should return all new commits since a given commit', () => __awaiter(this, void 0, void 0, function* () {
            let since;
            for (let i = 0; i < 4; i++) {
                yield fs.appendFile('test.txt', `${i}\n`);
                yield child_process_1.execFile('git', ['add', '.']);
                yield child_process_1.execFile('git', ['commit', '-m', `${i}`]);
                if (i === 1) {
                    since = yield git_1.getHead();
                }
            }
            const commitSequence = yield git_1.getNewCommits(since);
            assert.equal(commitSequence.isReversed, false);
            assert.equal(commitSequence.length, 2);
            assert.equal(commitSequence[0].message, '2');
            assert.equal(commitSequence[1].message, '3');
        }));
        it('should return 0 without any commits', () => __awaiter(this, void 0, void 0, function* () {
            const commitSequence = yield git_1.getNewCommits();
            assert.equal(commitSequence.length, 0);
            assert.equal(commitSequence.isReversed, false);
        }));
        describe('when HEAD is behind since', () => {
            it('should return all commits between HEAD and since in reverse', () => __awaiter(this, void 0, void 0, function* () {
                let since;
                for (let i = 0; i < 4; i++) {
                    yield fs.appendFile('test.txt', `${i}\n`);
                    yield child_process_1.execFile('git', ['add', '.']);
                    yield child_process_1.execFile('git', ['commit', '-m', i + '']);
                    if (i === 3) {
                        since = yield git_1.getHead();
                    }
                }
                yield child_process_1.execFile('git', ['checkout', 'HEAD^^']);
                const commitSequence = yield git_1.getNewCommits(since);
                assert.equal(commitSequence.isReversed, true);
                assert.equal(commitSequence.length, 2);
                assert.equal(commitSequence[0].message, '2');
                assert.equal(commitSequence[1].message, '3');
            }));
        });
        it('should throw an UnknownCommitError when the since commit is unknown', () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield git_1.getNewCommits(new git_1.Commit({ sha1: 'whatever' }));
                throw new assert.AssertionError({ message: 'Did not throw' });
            }
            catch (err) {
                if (!(err instanceof git_1.UnknownCommitError)) {
                    throw err;
                }
            }
        }));
    });
    describe('getHead', () => {
        it('should get the correct head when a head is present', () => __awaiter(this, void 0, void 0, function* () {
            for (let i = 0; i < 2; i++) {
                yield fs.appendFile('test.txt', `${i}\n`);
                yield child_process_1.execFile('git', ['add', '.']);
                yield child_process_1.execFile('git', ['commit', '-m', `${i}`]);
            }
            const commit = yield git_1.getHead();
            assert.equal(typeof commit.sha1, 'string');
        }));
        it('should get error when no HEAD exists', () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield git_1.getHead();
                assert.fail();
            }
            catch (err) {
                if (!(err instanceof git_1.NoCommitsError)) {
                    throw err;
                }
            }
        }));
    });
    describe('isRevertCommit', () => {
        it('should find revert commits', () => __awaiter(this, void 0, void 0, function* () {
            yield fs.appendFile('test.txt', 'A');
            yield child_process_1.execFile('git', ['add', '.']);
            yield child_process_1.execFile('git', ['commit', '-m', 'Msg']);
            yield child_process_1.execFile('git', ['revert', 'HEAD']);
            const commitSequence = yield git_1.getNewCommits();
            assert(git_1.isRevertCommit(commitSequence[1].message));
        }));
        it('should just find revert commits', () => __awaiter(this, void 0, void 0, function* () {
            yield fs.appendFile('test.txt', 'A');
            yield child_process_1.execFile('git', ['add', '.']);
            yield child_process_1.execFile('git', ['commit', '-m', 'Msg']);
            const commitSequence = yield git_1.getNewCommits();
            assert(!git_1.isRevertCommit(commitSequence[0].message));
        }));
    });
    describe('parseGitLog', () => {
        it('should return empty array on empty log', () => {
            assert.deepEqual(git_1.parseGitLog(''), new git_1.CommitSequence());
        });
        it('should parse hash, message and merkel tasks', () => __awaiter(this, void 0, void 0, function* () {
            const commits = git_1.parseGitLog(yield fs.readFile(path.resolve(path.join(__dirname, '../../src/test/git_log_output.txt')), 'utf8'));
            for (const commit of commits) {
                for (const task of commit.tasks) {
                    delete task.commit;
                }
            }
            assert.deepEqual(commits, git_1.CommitSequence.from([
                {
                    sha1: 'c3555604ddfc24022508c5e1f9398c81f3b9b6fa',
                    message: 'Revert changes to user model\n\nThis reverts commit b9fb8f15176d958d42dba4f14e35f1bf71ec0be9\nand 900931208ae7dabfd9ede49a78d4a961aa8043ba.',
                    tasks: [
                        {
                            type: 'down',
                            migration: {
                                name: 'd9271b98-1a2e-445f-8363-783b2bee0ef0'
                            }
                        },
                        {
                            type: 'down',
                            migration: {
                                name: '694b8b6c-3aaa-4e4d-8d54-76515ba90617'
                            }
                        }
                    ]
                },
                {
                    sha1: 'b9fb8f15176d958d42dba4f14e35f1bf71ec0be9',
                    message: 'Change the User model',
                    tasks: [
                        {
                            type: 'up',
                            migration: {
                                name: '694b8b6c-3aaa-4e4d-8d54-76515ba90617'
                            }
                        },
                        {
                            type: 'up',
                            migration: {
                                name: 'd9271b98-1a2e-445f-8363-783b2bee0ef0'
                            }
                        }
                    ]
                },
                {
                    sha1: '900931208ae7dabfd9ede49a78d4a961aa8043ba',
                    message: 'Initial Commit',
                    tasks: []
                }
            ]));
        }));
    });
    describe('getTasksForNewCommit', () => {
        it('should find new migrations', () => __awaiter(this, void 0, void 0, function* () {
            yield git_1.addGitHook();
            yield fs.mkdir('migrations');
            yield fs.appendFile('migrations/text.txt', 'ABC');
            yield child_process_1.execFile('git', ['add', '.']);
            const tasks = yield git_1.getTasksForNewCommit('migrations');
            assert.deepEqual(tasks, [
                {
                    migration: {
                        name: 'text'
                    },
                    type: 'up'
                }
            ]);
        }));
    });
    after(() => {
        return del('.git');
    });
});
//# sourceMappingURL=git.test.js.map