"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
const migration_1 = require("../migration");
const git_1 = require("../git");
const path = require("path");
const fs = require("mz/fs");
const del = require("del");
const assert = require("assert");
const os_1 = require("os");
const child_process_1 = require("mz/child_process");
const repo = path.join(os_1.tmpdir(), 'merkel_test_api');
describe('index', () => {
    before(() => __awaiter(this, void 0, void 0, function* () {
        try {
            yield fs.access(repo);
        }
        catch (err) {
            yield fs.mkdir(repo);
        }
        process.chdir(repo);
        yield del('*', { dot: true });
    }));
    describe('isMerkelRepository', () => {
        it('should find merkel repositorys by the .merkelrc.json', () => __awaiter(this, void 0, void 0, function* () {
            assert(!(yield index_1.isMerkelRepository()), 'a repo without config was thought to be a merkel repository');
            yield index_1.createConfig({
                migrationDir: 'test',
                migrationOutDir: 'test_out'
            });
            assert(yield index_1.isMerkelRepository(), 'a repo with config was thought to be no merkel repository');
        }));
    });
    describe('generate', () => {
        beforeEach(() => __awaiter(this, void 0, void 0, function* () {
            yield del('*', { dot: true });
        }));
        it('should generate typescript when tsconfig.json is present', () => __awaiter(this, void 0, void 0, function* () {
            yield fs.writeFile('tsconfig.json', `
                {
                    "compilerOptions": {
                        "target": "es6"
                    }
                }
            `);
            yield index_1.generate({ migrationDir: 'test', name: 'test' }, index_1.SILENT_LOGGER);
            const content = yield fs.readFile('test/test.ts', 'utf8');
            assert(content.includes('async'), 'no typescript migration was created.');
        }));
        it('should generate typescript without async when told to', () => __awaiter(this, void 0, void 0, function* () {
            yield fs.writeFile('tsconfig.json', `
                {
                    "compilerOptions": {
                        "target": "es3"
                    }
                }
            `);
            yield index_1.generate({ migrationDir: 'test', name: 'test' }, index_1.SILENT_LOGGER);
            const content = yield fs.readFile('test/test.ts', 'utf8');
            assert(!content.includes('async'), 'async keyword was used for target < ES6.');
        }));
        it('should generate javascript when no tsconfig.json is present', () => __awaiter(this, void 0, void 0, function* () {
            yield index_1.generate({ migrationDir: 'test', name: 'test' }, index_1.SILENT_LOGGER);
            try {
                yield fs.access('test/test.js');
            }
            catch (err) {
                throw new assert.AssertionError({
                    message: 'no javascript migration was created.'
                });
            }
        }));
        it('should render a template, if given', () => __awaiter(this, void 0, void 0, function* () {
            const template = `
                export function test() {
                }
            `;
            yield fs.writeFile('template.js', template);
            yield index_1.generate({
                migrationDir: 'test',
                name: 'test.js',
                template: 'template.js'
            }, index_1.SILENT_LOGGER);
            const content = yield fs.readFile('test/test.js', 'utf8');
            assert.equal(content, template);
        }));
        it('should throw if migration file would be overwritten', () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield index_1.createMigrationDir('test');
                yield fs.writeFile('test/test.js', `
                    export function up() {
                    }
                `);
                yield index_1.generate({
                    migrationDir: 'test',
                    name: 'test'
                }, index_1.SILENT_LOGGER);
                throw new assert.AssertionError({
                    message: 'it did not throw although migration was already present'
                });
            }
            catch (err) {
                if (!(err instanceof index_1.MigrationAlreadyExistsError)) {
                    throw err;
                }
            }
        }));
        it('should not crash if template does not exist', () => __awaiter(this, void 0, void 0, function* () {
            yield index_1.createMigrationDir('test');
            yield index_1.generate({
                migrationDir: 'test',
                name: 'test',
                template: 'none'
            }, index_1.SILENT_LOGGER);
        }));
    });
    describe('generateCommitMsg', () => {
        beforeEach(() => __awaiter(this, void 0, void 0, function* () {
            yield del('*', { dot: true });
        }));
        it('should warn if is a revert commit', () => __awaiter(this, void 0, void 0, function* () {
            yield child_process_1.execFile('git', ['init']);
            yield fs.writeFile('msgfile', 'Revert: Commit 91ba39f');
            let warned = false;
            yield index_1.prepareCommitMsg('msgfile', 'migrations', {
                warn: () => {
                    warned = true;
                },
                error: () => undefined,
                log: () => undefined
            });
            assert(warned, 'the generator has not echoed a warning');
        }));
        it('should append a new migration to msg', () => __awaiter(this, void 0, void 0, function* () {
            yield child_process_1.execFile('git', ['init']);
            yield fs.mkdir('migrations');
            yield fs.writeFile('migrations/test.js', 'export function up() {}');
            yield child_process_1.execFile('git', ['add', 'migrations/test.js']);
            yield fs.writeFile('msgfile', '');
            yield index_1.prepareCommitMsg('msgfile', 'migrations');
            const content = yield fs.readFile('msgfile', 'utf8');
            assert(content.includes('[merkel up test]'), 'migration was not correctly added to commit message');
            assert(!content.includes('[merkel down'), 'migration was not correctly added to commit message');
        }));
    });
    describe('Status', () => {
        describe('toString', () => {
            it('should print the repositories current status', () => {
                const status = new index_1.Status();
                status.head = new git_1.Commit({ sha1: '0c0302301' });
                status.newCommits = new git_1.CommitSequence();
                const tasks = new migration_1.TaskList();
                tasks.push(new migration_1.Task({
                    type: 'up',
                    migration: new migration_1.Migration({ name: 'user' })
                }));
                const commit = new git_1.Commit({
                    sha1: '0c0302301',
                    tasks: tasks
                });
                status.newCommits.push(commit);
                const output = status.toString();
                assert(output.includes('Last migration:      No migration run yet'));
                assert(output.includes('<unknown commit>'));
                assert(output.includes('1 pending migration:'));
                assert(output.includes('▲ UP   user'));
            });
            it('should print the last task', () => {
                const status = new index_1.Status();
                status.head = new git_1.Commit({ sha1: '0c0302301', message: 'top' });
                status.lastTask = new migration_1.Task({
                    type: 'up',
                    appliedAt: new Date(0),
                    commit: new git_1.Commit({
                        sha1: '9913944',
                        message: 'initial'
                    }),
                    head: new git_1.Commit({
                        sha1: '9991920',
                        message: 'header'
                    }),
                    migration: new migration_1.Migration({ name: 'user' })
                });
                status.newCommits = new git_1.CommitSequence();
                const output = status.toString();
                assert(output.includes('▲ UP   user'));
                assert(output.includes('Thu Jan 01 1970'));
                assert(output.includes('initial'));
                assert(output.includes('header'));
                assert(output.includes('‖'));
                assert(output.includes('top'));
            });
        });
    });
    after(() => __awaiter(this, void 0, void 0, function* () {
        yield del('*', { dot: true });
    }));
});
//# sourceMappingURL=index.test.js.map