"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const fs = require("mz/fs");
const path = require("path");
const pg = require("pg");
const migration_1 = require("../migration");
const git_1 = require("../git");
const postgres_1 = require("../adapters/postgres");
const del = require("del");
const os_1 = require("os");
const sinon = require("sinon");
const repo = path.join(os_1.tmpdir(), 'merkel_test_api');
describe('migration', () => {
    describe('Migration', () => {
        before(() => __awaiter(this, void 0, void 0, function* () {
            try {
                yield fs.access(repo);
            }
            catch (err) {
                yield fs.mkdir(repo);
            }
            process.chdir(repo);
            yield del('*', { dot: true });
        }));
        describe('getPath()', () => {
            it('should calculate the right path', () => __awaiter(this, void 0, void 0, function* () {
                yield fs.mkdir(path.join(repo, 'migrations'));
                yield fs.writeFile(path.join('migrations', 'test.js'), 'up');
                const migration = new migration_1.Migration({ name: 'test' });
                assert.equal(yield migration.getPath('migrations'), path.resolve('migrations/test.js'));
            }));
            it('should throw MigrationNotFoundError for not found migrations', () => __awaiter(this, void 0, void 0, function* () {
                const migration = new migration_1.Migration();
                try {
                    yield migration.getPath('migrations');
                }
                catch (err) {
                    if (!(err instanceof migration_1.MigrationNotFoundError)) {
                        throw err;
                    }
                }
            }));
        });
        after(() => __awaiter(this, void 0, void 0, function* () {
            yield del('*', { dot: true });
        }));
    });
    describe('TaskList', () => {
        describe('execute()', () => {
            it('should run all its tasks', () => __awaiter(this, void 0, void 0, function* () {
                const taskList = new migration_1.TaskList();
                const task = new migration_1.Task();
                taskList.push(task);
                let stub;
                try {
                    stub = sinon.stub(task, 'execute');
                    taskList.execute(null, null, null);
                }
                finally {
                    stub.restore();
                }
                sinon.assert.calledOnce(stub);
            }));
        });
        describe('toString()', () => {
            it('should return a string of merkel commands', () => __awaiter(this, void 0, void 0, function* () {
                const taskList = migration_1.TaskList.from([
                    new migration_1.Task({ type: 'down', migration: new migration_1.Migration({ name: 'a4bdf4cc-f7ed-4785-8bdb-2abfb5ed2025' }) }),
                    new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'e55d537d-ad67-40e6-9ef1-50320e530676' }) })
                ]);
                assert.equal(taskList.toString(), '[merkel up e55d537d-ad67-40e6-9ef1-50320e530676]\n[merkel down a4bdf4cc-f7ed-4785-8bdb-2abfb5ed2025]');
            }));
            it('should wrap long commands', () => __awaiter(this, void 0, void 0, function* () {
                const taskList = migration_1.TaskList.from([
                    new migration_1.Task({ type: 'down', migration: new migration_1.Migration({ name: 'a4bdf4cc-f7ed-4785-8bdb-2abfb5ed2025' }) }),
                    new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'e55d537d-ad67-40e6-9ef1-50320e530676' }) }),
                    new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: '15ca72e9-241c-4fcd-97ad-aeb367403a27' }) }),
                    new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'e55d537d-ad67-40e6-9ef1-50320e530676' }) })
                ]);
                assert.equal(taskList.toString(), [
                    '[',
                    '  merkel up',
                    '  e55d537d-ad67-40e6-9ef1-50320e530676',
                    '  15ca72e9-241c-4fcd-97ad-aeb367403a27',
                    '  e55d537d-ad67-40e6-9ef1-50320e530676',
                    ']',
                    '[merkel down a4bdf4cc-f7ed-4785-8bdb-2abfb5ed2025]'
                ].join('\n'));
            }));
        });
    });
    describe('Task', () => {
        describe('invert()', () => {
            it('should return a new tasklist that is the inverse of the task', () => {
                const task = new migration_1.Task({ type: 'up' });
                const inverted = task.invert();
                assert.notEqual(task, inverted);
                assert.equal(inverted.type, 'down');
            });
        });
        describe('execute()', () => {
            let client;
            let adapter;
            before(() => __awaiter(this, void 0, void 0, function* () {
                adapter = new postgres_1.PostgresAdapter(process.env.MERKEL_DB, pg);
                yield adapter.init();
                client = new pg.Client(process.env.MERKEL_DB);
                yield new Promise((resolve, reject) => client.connect(err => err ? reject(err) : resolve()));
                yield client.query('DROP TABLE IF EXISTS new_table');
                yield client.query('TRUNCATE TABLE merkel_meta RESTART IDENTITY');
            }));
            after(() => client.end());
            it('should execute an up task', () => __awaiter(this, void 0, void 0, function* () {
                const task = new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'test_migration' }) });
                const head = new git_1.Commit({ sha1: 'HEADCOMMITSHA1' });
                const trigger = new git_1.Commit({ sha1: 'TRIGGERCOMMITSHA1' });
                yield task.execute(__dirname + '/migrations', adapter, head, trigger);
                // Check if table was created
                const { rows } = yield client.query(`SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE table_name = 'new_table'`);
                assert.equal(rows.length, 1);
            }));
            it('should throw a MigrationNotFoundError if the migration is not existent', () => __awaiter(this, void 0, void 0, function* () {
                const task = new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'not_found' }) });
                const head = new git_1.Commit({ sha1: 'HEADCOMMITSHA1' });
                const trigger = new git_1.Commit({ sha1: 'TRIGGERCOMMITSHA1' });
                try {
                    yield task.execute(__dirname + '/migrations', adapter, head, trigger);
                    throw new assert.AssertionError({
                        message: 'No MigrationNotFoundError was thrown.'
                    });
                }
                catch (err) {
                    if (!(err instanceof migration_1.MigrationNotFoundError)) {
                        throw err;
                    }
                }
            }));
            it('should throw a TaskTypeNotFoundError if the migration has no up or down function', () => __awaiter(this, void 0, void 0, function* () {
                const task = new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'no_up' }) });
                const head = new git_1.Commit({ sha1: 'HEADCOMMITSHA1' });
                const trigger = new git_1.Commit({ sha1: 'TRIGGERCOMMITSHA1' });
                try {
                    yield task.execute(__dirname + '/migrations', adapter, head, trigger);
                    throw new assert.AssertionError({
                        message: 'No TaskTypeNotFoundError was thrown.'
                    });
                }
                catch (err) {
                    if (!(err instanceof migration_1.TaskTypeNotFoundError)) {
                        throw err;
                    }
                }
            }));
            it('should throw a MigrationExecutionError when the migration returns a rejected promise', () => __awaiter(this, void 0, void 0, function* () {
                const task = new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'error_async' }) });
                const head = new git_1.Commit({ sha1: 'HEADCOMMITSHA1' });
                const trigger = new git_1.Commit({ sha1: 'TRIGGERCOMMITSHA1' });
                try {
                    yield task.execute(__dirname + '/migrations', adapter, head, trigger);
                    throw new assert.AssertionError({
                        message: 'No MigrationExecutionError was thrown.'
                    });
                }
                catch (err) {
                    if (!(err instanceof migration_1.MigrationExecutionError)) {
                        throw err;
                    }
                }
            }));
            it('should throw a MigrationExecutionError when the migration throws sync', () => __awaiter(this, void 0, void 0, function* () {
                const task = new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'error_sync' }) });
                const head = new git_1.Commit({ sha1: 'HEADCOMMITSHA1' });
                const trigger = new git_1.Commit({ sha1: 'TRIGGERCOMMITSHA1' });
                try {
                    yield task.execute(__dirname + '/migrations', adapter, head, trigger);
                    throw new assert.AssertionError({
                        message: 'No Migration ExecutionError was thrown.'
                    });
                }
                catch (err) {
                    if (!(err instanceof migration_1.MigrationExecutionError)) {
                        throw err;
                    }
                }
            }));
            it('should throw a MigrationExecutionError when the migration would crash the process', () => __awaiter(this, void 0, void 0, function* () {
                const task = new migration_1.Task({ type: 'up', migration: new migration_1.Migration({ name: 'error_crash' }) });
                const head = new git_1.Commit({ sha1: 'HEADCOMMITSHA1' });
                const trigger = new git_1.Commit({ sha1: 'TRIGGERCOMMITSHA1' });
                let listener;
                try {
                    listener = process.listeners('uncaughtException').pop();
                    process.removeListener('uncaughtException', listener);
                    yield task.execute(__dirname + '/migrations', adapter, head, trigger);
                    throw new assert.AssertionError({
                        message: 'No Migration ExecutionError was thrown.'
                    });
                }
                catch (err) {
                    if (!(err instanceof migration_1.MigrationExecutionError)) {
                        throw err;
                    }
                }
                finally {
                    process.addListener('uncaughtException', listener);
                }
            }));
        });
        describe('toString', () => {
            it('should throw if the task type is unknown', () => __awaiter(this, void 0, void 0, function* () {
                const task = new migration_1.Task({ type: 'd' });
                try {
                    task.toString();
                    throw new assert.AssertionError({
                        message: 'Task did not throw error on toString with wrong type'
                    });
                }
                catch (err) {
                    if (!(err instanceof migration_1.UnknownTaskTypeError)) {
                        throw err;
                    }
                }
            }));
        });
    });
});
//# sourceMappingURL=migration.test.js.map